import { copyFile, mkdir, readFile, writeFile } from 'fs/promises'
import subsetFont from 'subset-font'
import { JSDOM } from 'jsdom'
import { createHash } from 'crypto';

let html = await readFile('src/index.html', 'utf8');
const dom = new JSDOM(html);
const doc = dom.window.document;

const fonts = {
    merriweather: {
        font: 'merriweather-v27-latin',
        weights: {
            900: () => doc.querySelector('h1').firstChild.textContent,
            300: '[role="doc-subtitle"]',
            400: 'h2'
        }
    },
    inter: {
        font: 'inter-v7-latin',
        weights: {
            400: '.box',
            700: '.box b'
        }
    }
};

await mkdir('dist/static', { recursive: true });

for (const [name, f] of Object.entries(fonts)) {
    for (const [weight, selector] of Object.entries(f.weights)) {
        const path = `assets/${f.font}-${weight}.woff2`;
        const words = typeof selector === 'string' ? Array.from(doc.querySelectorAll(selector)).map(x => x.textContent).join(' ') : selector();

        const subset = await subsetFont(await readFile(path), words, { targetFormat: 'woff2' });
        html = html.replace(`FONT_${name}_${weight}`, 'data:font/woff2;base64,' + subset.toString('base64'));
    }
}

{
    const laputa = await readFile('assets/laputa.webp');
    const outName = `static/laputa.${makeHash(laputa)}.webp`;
    await writeFile('dist/' + outName, laputa);
    const svg = await readFile('assets/laputa.svg');
    html = html.replace('LAPUTA_WEBP', outName).replace('LAPUTA_SVG', 'data:image/svg+xml;base64,' + svg.toString('base64'));
}

await writeFile('dist/index.html', html);
await copyFile('src/favicon.ico', 'dist/favicon.ico');

function makeHash(input) {
    const hash = createHash('sha256');
    hash.update(input);
    return hash.digest('hex').substring(0, 8);
}
